/*
 * Project: VizKit
 * Version: 3.3.8
 
 * Date: 20150503
 * File: TrackLyricsActor.cpp
 *
 */

/***************************************************************************

Copyright (c) 2004-2015 Heiko Wichmann (http://www.imagomat.de/vizkit)


This software is provided 'as-is', without any expressed or implied warranty. 
In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; 
   you must not claim that you wrote the original software. 
   If you use this software in a product, an acknowledgment 
   in the product documentation would be appreciated 
   but is not required.

2. Altered source versions must be plainly marked as such, 
   and must not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source distribution.

 ***************************************************************************/

#include "TrackLyricsActor.h"
#include "TrackLyrics.h"
#include "VisualNotification.h"
#include "VisualErrorHandling.h"
#include "VisualConfiguration.h"
#include "VisualDataStore.h"
#include "VisualPreferences.h"
#include "VisualString.h"
#include "VisualStringStyle.h"
#include "VisualImage.h"
#include "VisualStyledString.h"
#include "VisualPlayerState.h"


using namespace VizKit;


TrackLyricsActor::TrackLyricsActor()
{
	strcpy(actorName, "TRACKLYRICS");
	state = kVisActOn;
    trackLyrics = new TrackLyrics;
	textureOfCurrentTrackLyricsIsAvailable = false;
}


TrackLyricsActor::~TrackLyricsActor()
{
    trackLyrics->clear();
	delete trackLyrics;
}


void TrackLyricsActor::init()
{
	VisualNotification::registerNotification(this, kGetTrackLyricsStringStyleMsg);
    VisualNotification::registerNotification(this, kTrackLyricsStringMsg);
	VisualNotification::registerNotification(this, kAudioPlayTrackChangedEvt);
	VisualNotification::registerNotification(this, kAudioPlayStoppedEvt);
	VisualNotification::registerNotification(this, kAudioPlayPausedEvt);
	VisualNotification::registerNotification(this, kAudioPlayResumedEvt);
	VisualNotification::registerNotification(this, kAudioPlayReachedFadeOutTimeEvt);
    VisualNotification::registerNotification(this, kCanvasReshapeEvt);
}


void TrackLyricsActor::show(const VisualPlayerState& visualPlayerState)
{
	if ((this->textureOfCurrentTrackLyricsIsAvailable == true) && (this->state != kVisActOff))
    {
        uint32 anElapsedAudioTime = visualPlayerState.getElapsedAudioTime();
        uint32 aRemainingAudioTime = visualPlayerState.getRemainingAudioTime();
		this->trackLyrics->show(anElapsedAudioTime, aRemainingAudioTime);
	}
}


void TrackLyricsActor::handleNotification(VisualNotification& aNotification)
{

	// VisualActor::handleNotification(aNotification); // debug
	
	VisualString trackLyricsStr;

	VisualNotificationKey notificationKey = aNotification.getKey();
	
	switch (notificationKey) {
	
		case kGetTrackLyricsStringStyleMsg:
			{
				VisualStringStyle lyricsStringStyle = this->trackLyrics->getTrackLyricsStringStyle();
				aNotification.setObject(lyricsStringStyle);				
			}
			break;
            
        case kTrackLyricsStringMsg:
            {
                VisualStyledString *aStyledLyricsString = dynamic_cast<VisualStyledString*>(aNotification.getObject());
                if (aStyledLyricsString->getNumberOfNonWhitespaceCharacters() > 0)
                {
                    this->trackLyrics->setTrackLyricsString(*aStyledLyricsString);
                    this->textureOfCurrentTrackLyricsIsAvailable = true;
                    bool aStartWithCurrentOpacityValueFlag = false;
                    this->trackLyrics->fadeIn(VisualPreferences::getValue(VisualPreferences::kFadeInTimeOnPlayInMS), aStartWithCurrentOpacityValueFlag);
                }
            }
			break;
			
		case kAudioPlayTrackChangedEvt:
			this->trackLyrics->clear();
			this->textureOfCurrentTrackLyricsIsAvailable = false;
			break;
            
        case kCanvasReshapeEvt:
            this->trackLyrics->refresh();
            break;
			
		case kAudioPlayStoppedEvt:
			this->trackLyrics->clear();
			this->textureOfCurrentTrackLyricsIsAvailable = false;
			break;
			
		case kAudioPlayPausedEvt:
            if (this->textureOfCurrentTrackLyricsIsAvailable)
            {
                this->trackLyrics->fadeOut(VisualPreferences::getValue(VisualPreferences::kFadeOutTimeOnPauseInMS), 0.15f);
            }
			break;
			
		case kAudioPlayResumedEvt:
            if (this->textureOfCurrentTrackLyricsIsAvailable)
            {
                bool aStartWithCurrentOpacityValueFlag = true;
                this->trackLyrics->fadeIn(VisualPreferences::getValue(VisualPreferences::kFadeInTimeOnResumeInMS), aStartWithCurrentOpacityValueFlag);
            }
			break;
			
		case kAudioPlayReachedFadeOutTimeEvt:
            if (this->textureOfCurrentTrackLyricsIsAvailable)
            {
                this->trackLyrics->fadeOut(VisualPreferences::getValue(VisualPreferences::kFadeOutTimeBeforeEndOfTrackInMS));
            }
			break;
			
		default:
			writeLog("unhandled Notification in TrackLyricsActor");
			break;
	}

}
