/*
 * Project: VizKit
 * Version: 3.3.8
 
 * Date: 20150503
 * File: VisualFontManager.h
 *
 */

/***************************************************************************

Copyright (c) 2004-2015 Heiko Wichmann (http://www.imagomat.de/vizkit)


This software is provided 'as-is', without any expressed or implied warranty. 
In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; 
   you must not claim that you wrote the original software. 
   If you use this software in a product, an acknowledgment 
   in the product documentation would be appreciated 
   but is not required.

2. Altered source versions must be plainly marked as such, 
   and must not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source distribution.

 ***************************************************************************/

#ifndef VisualFontManager_h
#define VisualFontManager_h


#include "VisualTypes.h"


#if TARGET_OS_MAC
#include <CoreFoundation/CoreFoundation.h>
#endif

#if TARGET_OS_WIN
#include <windows.h>
#endif

#include <vector>



#if TARGET_OS_WIN
		class fontItem {
		public:
			char name[64];
		};
#endif


namespace VizKit {

	/**
	 * Return values for explicit font activation.
	 */
	typedef enum {
		fontActivationUndefined = 0, /**< Undefined state. */
		fontActivationNoError, /**< No error. */
		fontActivationFontAlreadyAvailable, /**< The font was already available (no activation of the font resource has been necessary). */
		fontActivationUnableToActivateFont /**< Unable to activate font. */
	} VisualFontActivationResult;

	/**
	 * Management of font resources.
	 */
	class VisualFontManager {

	public:

		/**
		 * Constructs a VisualFontManager. The VisualFontManager internally is a singleton. Returns a pointer to the initialized VisualFontManager.
		 * @return A pointer to the singleton instance.
		 */
		static VisualFontManager* getInstance(void);

		/**
		 * Disposes the VisualFontManager.
		 */	
		static void dispose(void);

		/**
		 * Answers the question whether a given font is available as private resource of the visualizer.
		 * @param fontName The name of the font.
		 * @return True if font is available as private resource of the visualizer, false if not.
		 * @remarks This function is a stub, custom tailored for VizKit and its packaging.
		 */
		bool fontIsAvailableAsVisualizerResource(const char* const fontName);

		/**
		 * Tries to activate font by loading a font resource.
		 * @param fontName The name of the font.
		 * @return The result of the operation.
		 */
		VisualFontActivationResult activateFont(const char* const fontName);
		
#if TARGET_OS_WIN
		static int CALLBACK EnumFontFamExProc2(
						ENUMLOGFONTEX *lpelfe,    // logical-font data
						NEWTEXTMETRICEX *lpntme,  // physical-font data
						DWORD FontType,           // type of font
						LPARAM lParam             // application-defined data
		);
		void resetFontVectorIterIndex(void);
		void gatherFonts(void);
		bool getNextFontName(char* fontName);
#endif

		/**
		 * Sorts the gathered fonts by name.
		 */
		void sortFontsByName(void);

		/**
		 * Adds a font name to the gathered list of font names.
		 * @param fontName The name of the font to add.
		 */
		void addFont(const char* const fontName);


#if TARGET_OS_WIN
		/**
		 * The function queries the system to determine if the font is installed. Windows-only.
		 * @param currHdc Pointer to the current device.
		 * @param fontName The name of the font.
		 * @return 1 if font is installed, 0 if font is not installed.
		 */
		uint8 fontIsInstalled(HDC currHdc, const char* const fontName);
#endif


	private:
	
		/** The constructor. VisualFontManager is a singleton class. The constructor is private. New instance of class can only be created internally. */
		VisualFontManager();

		/** The destructor. VisualFontManager is a singleton class. The destructor is private. Instance of class can only be destructed internally. */
		~VisualFontManager();

		/**
		 * Copy constructor.
		 * @param other Another VisualFontManager.
		 * @remarks Explicitely declared in private section and not implemented to enforce uniqueness of singleton pattern.
		 */
		VisualFontManager(const VisualFontManager& other);

		/**
		 * Assignment operator.
		 * @remarks Explicitely declared in private section and not implemented to enforce uniqueness of singleton pattern.
		 */			
		VisualFontManager& operator=(const VisualFontManager& other);

		/** VisualFontManager is a singleton class. Pointer to private instance is handled internally. */
		static VisualFontManager* theVisualFontManager;

#if TARGET_OS_WIN
		std::vector<fontItem*> FontVector;
		uint16 fontVectorIterIdx;
#endif

#if TARGET_OS_MAC
        
        bool fontIsAvailable(const char* const fontNameUTF8String);
        
        OSStatus activateCocoaFont(const char* const fontNameUTF8String);
#endif

	};
	
}

#endif /* VisualFontManager_h */
