/*
 * Project: VizKit
 * Version: 3.3.8
 
 * Date: 20150503
 * File: VisualStyledString.cpp
 *
 */

/***************************************************************************

Copyright (c) 2004-2015 Heiko Wichmann (http://www.imagomat.de/vizkit)


This software is provided 'as-is', without any expressed or implied warranty. 
In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; 
   you must not claim that you wrote the original software. 
   If you use this software in a product, an acknowledgment 
   in the product documentation would be appreciated 
   but is not required.

2. Altered source versions must be plainly marked as such, 
   and must not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source distribution.

 ***************************************************************************/

#include "VisualStyledString.h"
#include "VisualGraphics.h"
#include "VisualGraphicsCore.h"
#include "VisualErrorHandling.h"


#if TARGET_OS_MAC
#import <Cocoa/Cocoa.h>
#endif
// file type of VisualStyledString.cpp is set to "Objective-C++ source"


#if TARGET_OS_WIN
#include <windows.h>
#include <locale>
#include <codecvt>
#include <string>
#endif

using namespace VizKit;


VisualStyledString::VisualStyledString() 
{
	// NULL
}


VisualStyledString::~VisualStyledString() 
{
	// NULL
}


VisualStyledString::VisualStyledString(const VisualStyledString& other) : VisualString(other) 
{
	this->stringStyle = other.stringStyle;
}


VisualStyledString& VisualStyledString::operator=(const VisualStyledString& other) 
{

	if (this == &other) return *this;

	VisualString::operator=(other);

	this->stringStyle = other.stringStyle;
	
	return *this;

}


VisualStyledString* VisualStyledString::clone(void) const 
{
	return new VisualStyledString(*this);
}


void VisualStyledString::initWithVisualString(const VisualString& anUnstyledString) 
{
	VisualString::initWithUTF16String(const_cast<VisualString&>(anUnstyledString).getUtf16Representation());
}


void VisualStyledString::setStyle(const VisualStringStyle& aStringStyle) 
{
	this->stringStyle = aStringStyle;
}


VisualStringStyle& VisualStyledString::getStyle() 
{
	return this->stringStyle;
}


PixelRect VisualStyledString::bitmapPixelRect(size_t aMaxPixelWidth, size_t aMaxPixelHeight)
{
    PixelRect aPixelRect;
	aPixelRect.width = 0;
	aPixelRect.height = 0;

#if TARGET_OS_MAC
    
    VisualGraphics* theVisualGraphics = VisualGraphics::getInstance();
    float aScreenScaleFactor = theVisualGraphics->getScreenScaleFactor();
    // Retina display: To ensure correct multi-line calculations
    // we need to pass logical dimensions to NSAttributedString's boundingRectWithSize:options:
    aMaxPixelWidth /= aScreenScaleFactor;
    aMaxPixelHeight /= aScreenScaleFactor;
    
    NSGraphicsContext* aContext = [NSGraphicsContext currentContext];
    [aContext saveGraphicsState];
    
    VisualStringStyle& aStringStyle = this->getStyle();
    
    char *aFontNameUTF8CharStr = aStringStyle.fontNameStr;
    double aFontSize = aStringStyle.fontSize;
    const CFStringRef aCFStringRef = this->getCharactersPointer();
    
    NSString *aFontName = [NSString stringWithUTF8String:aFontNameUTF8CharStr];
    
    NSMutableDictionary* aFontAttributesDict = [NSMutableDictionary dictionary];
    [aFontAttributesDict setObject: [NSFont fontWithName:aFontName size:(CGFloat)aFontSize] forKey:NSFontAttributeName];
    [aFontAttributesDict setObject: [NSColor colorWithDeviceRed:aStringStyle.fontColor.red green:aStringStyle.fontColor.green blue:aStringStyle.fontColor.blue alpha:1.0f] forKey: NSForegroundColorAttributeName];
    
    NSMutableAttributedString* anAttributedString = [[NSMutableAttributedString alloc] initWithString:(NSString*)aCFStringRef attributes:aFontAttributesDict];
    
    NSSize anAttributedStringConstraintFrameSize = NSMakeSize(0.0, 0.0);
    if (aMaxPixelWidth > 0)
    {
        anAttributedStringConstraintFrameSize.width = aMaxPixelWidth;
    }
    if (aMaxPixelHeight > 0)
    {
        anAttributedStringConstraintFrameSize.height = aMaxPixelHeight;
    }
    NSRect aTextFrameRect = [anAttributedString boundingRectWithSize:anAttributedStringConstraintFrameSize options:(NSStringDrawingUsesLineFragmentOrigin | NSStringDrawingOneShot)];
    aPixelRect.width = (ceilf)(aTextFrameRect.size.width);
    aPixelRect.height = (ceilf)(aTextFrameRect.size.height);
    
    // Retina display uses pixel metrics
    // text frame size is logical size, therefore half of actual image pixel size
    aPixelRect.width *= aScreenScaleFactor;
    aPixelRect.height *= aScreenScaleFactor;
    
    // TEST (start)
    /*
	NSSize answer = NSZeroSize ;
    NSSize size = NSMakeSize(aMaxPixelWidth, aMaxPixelHeight) ;
    NSTextContainer *textContainer = [[NSTextContainer alloc] initWithContainerSize:size] ;
    NSTextStorage *textStorage = [[NSTextStorage alloc] initWithAttributedString:anAttributedString] ;
    NSLayoutManager *layoutManager = [[NSLayoutManager alloc] init] ;
    [layoutManager addTextContainer:textContainer] ;
    [textStorage addLayoutManager:layoutManager] ;
    [layoutManager setHyphenationFactor:0.0] ;
    // NSLayoutManager is lazy, so we need the following kludge to force layout:
    [layoutManager glyphRangeForTextContainer:textContainer] ;
    
    answer = [layoutManager usedRectForTextContainer:textContainer].size ;
    
    
    NSLog(@"aPixelRect.width: %ld, height: %ld, answer.width: %f, answer.height: %f", aPixelRect.width, aPixelRect.height, answer.width, answer.height);
    
    [textStorage release] ;
    [textContainer release] ;
    [layoutManager release] ;
    
    */
    // TEST (end)
    
    
    //this->stringStyle.fontSize = aFontSize;
    //this->stringStyle.fontSize = [anAttributedString
    
    [anAttributedString release];
    
    [aContext restoreGraphicsState];

#endif

#if TARGET_OS_WIN

	bool success = true;
	
	LOGFONT aFontDefinition;
	uint32 stringSizeWidth = 0;
	uint32 stringSizeHeight = 0;
	
	char errStr[512];

	UINT textFormat = (DT_EXPANDTABS|DT_NOPREFIX);
	VisualStringStyle aCurrentStringStyle = this->stringStyle;
	if (aCurrentStringStyle.horizontalAlignment == kLeftAligned) 
	{
		textFormat |= DT_LEFT;
	} 
	else if (aCurrentStringStyle.horizontalAlignment == kCenterAligned)
	{
		textFormat |= DT_CENTER;
	} 
	else if (aCurrentStringStyle.horizontalAlignment == kRightAligned) 
	{
		textFormat |= DT_RIGHT;
	}

	VisualGraphics* theVisualGraphics = VisualGraphics::getInstance();
	VisualGraphicsCore* theVisualGraphicsCore;
	theVisualGraphicsCore = VisualGraphicsCore::getInstance();
	
	HDC hdcTemp = CreateCompatibleDC(theVisualGraphicsCore->getWindowDC());
	
	ZeroMemory(&aFontDefinition, sizeof(LOGFONT));
	aFontDefinition.lfHeight = MulDiv((int)this->stringStyle.fontSize, GetDeviceCaps(hdcTemp, LOGPIXELSY), 72);
	aFontDefinition.lfWidth = 0;
	aFontDefinition.lfEscapement = 0;
	aFontDefinition.lfOrientation = 0;
	aFontDefinition.lfWeight = FW_DONTCARE;
	aFontDefinition.lfItalic = FALSE;
	aFontDefinition.lfUnderline = FALSE;
	aFontDefinition.lfStrikeOut = FALSE;
	aFontDefinition.lfCharSet = DEFAULT_CHARSET;
	aFontDefinition.lfOutPrecision = OUT_OUTLINE_PRECIS;
	aFontDefinition.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	aFontDefinition.lfQuality = ANTIALIASED_QUALITY;
	aFontDefinition.lfPitchAndFamily = DEFAULT_PITCH;
	if (strlen(this->stringStyle.fontNameStr) > 32) {
		sprintf(errStr, "length of fontName must not exceed 32 characters but is %d in file: %s (line: %d) [%s])", strlen(this->stringStyle.fontNameStr), __FILE__, __LINE__, __FUNCTION__);
		writeLog(errStr);
		wcscpy(aFontDefinition.lfFaceName, L"Arial"); // fallback font name
	} else {
		std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>> converter;
		std::wstring wcFontName = converter.from_bytes(this->stringStyle.fontNameStr);
		wcscpy(aFontDefinition.lfFaceName, wcFontName.c_str()); // length of fontName must not exceed 32 characters (incl. null terminator)
	}
	
	HFONT aFont = CreateFontIndirect(&aFontDefinition);
	
	if (!aFont) {
		sprintf(errStr, "Err after CreateFontIndirect() in file: %s (line: %d) [%s])", __FILE__, __LINE__, __FUNCTION__);
		writeLog(errStr);
	}
	
	SelectObject(hdcTemp, aFont);
	uint8 red = (uint8)(stringStyle.fontColor.r * 255.0f);
	uint8 green = (uint8)(stringStyle.fontColor.g * 255.0f);
	uint8 blue = (uint8)(stringStyle.fontColor.b * 255.0f);
	SetTextColor(hdcTemp, RGB((BYTE)red, (BYTE)green, (BYTE)blue));

	//SetTextAlign(hdcTemp, TA_TOP | TA_LEFT);
	
	RECT textDrawRect;
	textDrawRect.top = 0;
	textDrawRect.left = 0;
	if (aMaxPixelWidth > 0)
	{
		textDrawRect.right = aMaxPixelWidth;
	}
	else
	{
		textDrawRect.right = theVisualGraphics->getCanvasPixelWidth();
	}
	if (aMaxPixelHeight > 0)
	{
		textDrawRect.bottom = aMaxPixelWidth;
	}
	else
	{
		textDrawRect.bottom = theVisualGraphics->getCanvasPixelHeight();
	}
	int drawTextResult = 0;
	wchar_t* stringValueRef = (wchar_t*)(this->getCharactersPointer());
	drawTextResult = DrawTextW(hdcTemp, stringValueRef, this->getNumberOfCharacters(), &textDrawRect, DT_CALCRECT|DT_WORDBREAK|textFormat);
	
	if (textDrawRect.bottom == 0) {
		sprintf(errStr, "Err (textDrawRect.bottom == 0) in file: %s (line: %d) [%s])", __FILE__, __LINE__, __FUNCTION__);
		writeLog(errStr);
	}
	if (drawTextResult == 0) {
		sprintf(errStr, "Err (drawTextResult == 0) in file: %s (line: %d) [%s])", __FILE__, __LINE__, __FUNCTION__);
		writeLog(errStr);
		return aPixelRect;
	}

	stringSizeWidth = textDrawRect.right - textDrawRect.left;
	stringSizeHeight = textDrawRect.bottom - textDrawRect.top;
	
	aPixelRect.width = stringSizeWidth;
	aPixelRect.height = stringSizeHeight;

	DeleteObject(aFont);
	DeleteDC(hdcTemp);
#endif

	//aCurrentStringStyle.fontSize = (double)aFontSize;
	//this->stringStyle = aCurrentStringStyle;


    return aPixelRect;
    
}
