/*
 * Project: VizKit
 * Version: 3.3.8
 
 * Date: 20150503
 * File: VisualUpdateManager.cpp
 *
 */

/***************************************************************************

Copyright (c) 2004-2015 Heiko Wichmann (http://www.imagomat.de/vizkit)


This software is provided 'as-is', without any expressed or implied warranty. 
In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; 
   you must not claim that you wrote the original software. 
   If you use this software in a product, an acknowledgment 
   in the product documentation would be appreciated 
   but is not required.

2. Altered source versions must be plainly marked as such, 
   and must not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source distribution.

 ***************************************************************************/

#include "VisualUpdateManager.h"
#include "VisualTypes.h"
#include "VisualErrorHandling.h"
#include "VisualPreferences.h"
#include "VisualConfiguration.h"
#include "VisualDataStore.h"

#if TARGET_OS_MAC
#include <Carbon/Carbon.h> // CFURLRef, GetMacOSStatusErrorString
#endif

#if TARGET_OS_WIN
#include <locale>
#include <codecvt>
#include <string>
#endif


using namespace VizKit;


VisualUpdateManager* VisualUpdateManager::theVisualUpdateManager = NULL;


VisualUpdateManager::VisualUpdateManager() {
	updateInformation = NULL;
}


VisualUpdateManager::~VisualUpdateManager() {
	if (updateInformation != NULL) {
		delete updateInformation;
	}
}


VisualUpdateManager* VisualUpdateManager::getInstance() {
    if (theVisualUpdateManager == NULL) {
		theVisualUpdateManager = new VisualUpdateManager;
    }
	if (theVisualUpdateManager == NULL) {
		writeLog("ERR: init theVisualUpdateManager failed");
	}
	return theVisualUpdateManager;
}


void VisualUpdateManager::dispose() {
	if (theVisualUpdateManager != NULL) {
		delete theVisualUpdateManager;
		theVisualUpdateManager = NULL;
	}
}


void VisualUpdateManager::checkForUpdate() {
	bool doCheck = VisualPreferences::getValue(VisualPreferences::kCheckForUpdate);
	if (doCheck) {
        const char* const aCurrentVersionInformationURL = VisualConfiguration::currentVersionInformationURL();
        if (strlen(aCurrentVersionInformationURL) > 0)
        {
            theVisualUpdateManager = VisualUpdateManager::getInstance();
            if (theVisualUpdateManager)
            {
                theVisualUpdateManager->updateInformation = VisualString::createWithContentsOfURL(aCurrentVersionInformationURL, VisualDataStore::updateInformationStringId);
            }
        }
	}
}


void VisualUpdateManager::getMoreInfoAboutAvailableUpdate() {
	const char* URLBytes = theVisualUpdateManager->infoURL.getUtf8Representation();
	theVisualUpdateManager = VisualUpdateManager::getInstance();
	theVisualUpdateManager->launchBrowserToOpenURL(URLBytes);
}


void VisualUpdateManager::doDownloadAvailableUpdate(void) {
	const char* URLBytes = theVisualUpdateManager->downloadURL.getUtf8Representation();
	theVisualUpdateManager = VisualUpdateManager::getInstance();
	theVisualUpdateManager->launchBrowserToOpenURL(URLBytes);
}


void VisualUpdateManager::launchBrowserToOpenURL(const char* const URL) {
#if TARGET_OS_MAC
	CFIndex length = strlen(URL);
	CFStringEncoding encoding = kCFStringEncodingASCII;
	CFURLRef baseURL = NULL;
	CFURLRef url = CFURLCreateWithBytes(kCFAllocatorDefault, (UInt8*)URL, length, encoding, baseURL);
	CFArrayCallBacks* callBacks = NULL;
	CFMutableArrayRef inURLs = CFArrayCreateMutable(kCFAllocatorDefault, 1, callBacks);
	CFArrayAppendValue(inURLs, url);
	LSRolesMask inRole = kLSRolesViewer;
	const AEKeyDesc* inAEParam = NULL;
	const LSApplicationParameters* inAppParams = NULL;
	ProcessSerialNumber* outPSNs = NULL;
	CFIndex inMaxPSNCount = 0;
	OSStatus osStatus = LSOpenURLsWithRole(inURLs, inRole, inAEParam, inAppParams, outPSNs, inMaxPSNCount);
    CFRelease(url);
    CFRelease(inURLs);
	if (osStatus == noErr) {
		printf("no Err\n");
	} else {
		printf("Err launch: %s\n", GetMacOSStatusErrorString(osStatus));
	}
#endif
#if TARGET_OS_WIN
	std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>> converter;
	std::wstring wcURL = converter.from_bytes(URL);
	ShellExecute(NULL, L"open", wcURL.c_str(), NULL, NULL, SW_SHOW);
#endif
}


bool VisualUpdateManager::evaluateUpdateInformation() {
	bool updateIsAvailable = false;
	theVisualUpdateManager = VisualUpdateManager::getInstance();
	VisualString delimiterVerticalLine;
	delimiterVerticalLine.initWithUTF8Buffer("|", 1);
    
    int count = 0;
    std::vector<VisualString> stringVector;
    if (theVisualUpdateManager)
    {
        stringVector = theVisualUpdateManager->updateInformation->splitWithDelimiter(delimiterVerticalLine);
        for (std::vector<VisualString>::iterator it = stringVector.begin(); it != stringVector.end(); it++) {
            if (count == 0) {
                theVisualUpdateManager->currentVersionString = *(it);
                theVisualUpdateManager->currentVersionString.trim();
            } else if (count == 1) {
                theVisualUpdateManager->infoURL = *(it);
                theVisualUpdateManager->infoURL.trim();
            } else if (count == 2) {
                theVisualUpdateManager->downloadURL = *(it);
                theVisualUpdateManager->downloadURL.trim();
            }
            count++;
        }
    }
    
	uint8 currMajorVersion = 0;
	uint8 currMinorVersion = 0;
	uint8 currReleaseRevision = 0;
	VisualString delimiterPoint;
	delimiterPoint.initWithUTF8Buffer(".", 1);
	std::vector<VisualString> versionStringVector = theVisualUpdateManager->currentVersionString.splitWithDelimiter(delimiterPoint);
	count = 0;
	for (std::vector<VisualString>::iterator it = versionStringVector.begin(); it != versionStringVector.end(); it++) {
		if (count == 0) {
			currMajorVersion = atoi(it->getUtf8Representation());
		} else if (count == 1) {
			currMinorVersion = atoi(it->getUtf8Representation());
		} else if (count == 2) {
			currReleaseRevision = atoi(it->getUtf8Representation());
		}
		count++;
	}
	if (currMajorVersion > VisualConfiguration::majorReleaseNumber()) {
		updateIsAvailable = true;
	} else if (currMajorVersion == VisualConfiguration::majorReleaseNumber()) {
		if (currMinorVersion > VisualConfiguration::minorReleaseNumber()) {
			updateIsAvailable = true;
		} else if (currMinorVersion == VisualConfiguration::minorReleaseNumber()) {
			if (currReleaseRevision > VisualConfiguration::releaseRevisionNumber()) {
				updateIsAvailable = true;
			}
		}
	}
	return updateIsAvailable;
}
