/*
 * Project: VizKit
 * Version: 3.3.8
 
 * Date: 20150503
 * File: VisualMainAction.h
 *
 */

/***************************************************************************

Copyright (c) 2004-2015 Heiko Wichmann (http://www.imagomat.de/vizkit)


This software is provided 'as-is', without any expressed or implied warranty. 
In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; 
   you must not claim that you wrote the original software. 
   If you use this software in a product, an acknowledgment 
   in the product documentation would be appreciated 
   but is not required.

2. Altered source versions must be plainly marked as such, 
   and must not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source distribution.

 ***************************************************************************/

#ifndef VisualMainAction_h
#define VisualMainAction_h

#include "VisualTypes.h"

#if TARGET_OS_MAC
#include <Carbon/Carbon.h> // CFDataRef
#endif

#if TARGET_OS_WIN
#include <windows.h> // HWND
#endif


// definitions from "iTunesAPI.h"
#if TARGET_OS_WIN
#ifndef VISUAL_PLATFORM_VIEW
#define VISUAL_PLATFORM_VIEW HWND
#endif
#ifndef VISUAL_PLATFORM_DATA
#define	VISUAL_PLATFORM_DATA LPCVOID
#endif
#else // TARGET_OS_MAC
#ifdef __OBJC__
@class NSView;
#else
struct NSView;
#endif
#ifndef VISUAL_PLATFORM_VIEW
#define VISUAL_PLATFORM_VIEW NSView*
#endif
#ifndef VISUAL_PLATFORM_DATA
#define	VISUAL_PLATFORM_DATA CFDataRef
#endif
#endif

namespace VizKit {

	/**
	 * Dispatches the events and messages passed in by iTunes with no internal state.
	 */
	class VisualMainAction {

	public:

		/**
		 * Updates the current timestamp of the timing engine.
		 */	
		static void updateTiming(void);

		/**
		 * Handles initialization.
		 * @param majorRev Major revision number. 1st part of version number in BCD.
		 * @param minorRev Minor and bug revision number. 2nd and 3rd part of version number share a byte.
		 */	
		static void setAppVersion(int majorRev, int minorAndBugRev);
        
        /**
		 * Stores the major and minor version of the Visualizer API version implemented by the host application.
		 * @param majorVersionNum The major version number of the Visualizer API version implemented by the host application.
		 * @param minorVersionNum The minor version number of the Visualizer API version implemented by the host application.
		 */	
        static void setVisualizerAPIVersion(int majorVersionNum, int minorVersionNum);

		/**
		 * Handles initialization.
		 * @param maxDataChannels The number of data channels (2 for stereo).
		 * @param numberOfWaveformEntries The number of waveform data entries.
		 * @param numberOfSpectrumEntries The number of spectrum data entries.
		 */	
		static void handleInitMessage(int maxDataChannels, int numberOfWaveformEntries, int numberOfSpectrumEntries);

		/**
		 * Passes the pointer of the platform specific view.
		 * @param aView A platform specific view.
		 */	
		static void setPlatformView(VISUAL_PLATFORM_VIEW aView);

		/**
		 * Handles the show windows message.
		 * @param isFullScreen True if fullscreen.
		 */
		static void handleShowWindowMessage(bool isFullScreen);

        /**
		 * Kicks off the worker thread for rendering on Mac.
		 */
#if TARGET_OS_MAC
        static void startShow(void);
#endif
        
		/**
		 * Handles the set window message.
		 * @param isFullScreen True if fullscreen.
		 */	
		static void handleSetWindowMessage(bool isFullScreen);

		/**
		 * Handles the pulse message.
		 * @param currentPositionInMS The current position in the audio track.
		 * @param timeStampID Numeric id.
		 * @param numberOfWaveformChannels The number of waveform channels.
		 * @param numberOfWaveformEntries The number of waveform data entries.
		 * @param waveformData The waveform data.
		 * @param numberOfSpectrumChannels The number of spectrum channels.
		 * @param numberOfSpectrumEntries The number of spectrum data entries.
		 * @param spectrumData The spectrum data.
		 */	
		static void handlePulseMessage(int currentPositionInMS, int timeStampID, int numberOfWaveformChannels, int numberOfWaveformEntries, char* waveformData, int numberOfSpectrumChannels, int numberOfSpectrumEntries, char* spectrumData);

		/**
		 * Resets the audio lab data for the play message (in case the audio track does not unpause).
		 */	
		static void resetVisualAudioLabDataIfNotPaused();

		/**
		 * Prepares rendering.
		 * @return False on any failure.
		 */	
		static bool prepareRenderAction(void);

		/**
		 * The rendering action executed with each kVisualPluginRenderMessage/kVisualPluginIdleMessage.
		 * @remarks Probably the most often called function.
		 */	
		static void renderAction(void);

		/**
		 * Finishes rendering.
		 */	
		static void finishRenderAction(void);

		/**
		 * Handles the hide window message.
		 */	
		static void handleHideWindowMessage(void);

		/**
		 * Called when iTunes stops playing audio.
		 * Gathered audio data is cleared when the audio stops playing.
		 */
		static void handleAudioPlayStoppedEvent(void);

		/**
		 * Called when iTunes starts playing audio.
		 */
		static void handleAudioPlayStartedEvent(bool trackDidChange);

		/**
		 * Updates the tracked play position.
		 */
		static void setTrackPlayPositionInMS(int trackPlayPosition);

		/**
		 * Called when iTunes starts showing the visualizer.
		 * @param isFullscreen Set to true if fullscreen.
		 */
		static void handleShowShouldStartMessage(bool isFullscreen);

		/**
		 * Called when rendering of visualizer plug-in is closed.
		 */
		static void closeThreads(void);

		/**
		 * Called when visualizer plug-in is unloaded.
		 */
		static void closePersistentStore(void);

		/**
		 * Called when canvas is resized.
		 */
		static void handleCanvasReshape(void);

		/**
		 * Handles a key press event.
		 * @param aKeyPressed Character value of the key pressed.
         * @return True if key press event was handled by visualizer, false if key press event was not handled by visualizer.
		 */
		static bool handleKeyPressEvent(char aKeyPressed);

		/**
		 * Opens the dialog UI for configuration options.
		 */
		static void showConfigurationDialog(void);

		/**
		 * Resets the currently set audio meta data.
		 */
		static void resetCurrAudioMetaData(void);

		/**
		 * Sets the name of the audio track.
		 */
		static void setAudioTrackName(const void* const audioTrackName, int audioTrackNameLength);

		/**
		 * Sets the artist name of the audio track.
		 */
		static void setAudioTrackArtistName(const void* const audioTrackArtistName, int audioTrackArtistNameLength);

		/**
		 * Sets the album name of the audio track.
		 */
		static void setAudioTrackAlbumName(const void* const audioTrackAlbumName, int audioTrackAlbumNameLength);

		/**
		 * Sets the name of the composer of the audio track.
		 */
		static void setAudioTrackComposer(const void* const audioTrackComposer, int audioTrackComposerLength);

		/**
		 * Sets the title of an audio stream.
		 */
		static void setAudioStreamTitle(const void* const audioStreamTitle, int audioStreamTitleLength);

		/**
		 * Sets the message of an audio stream.
		 */
		static void setAudioStreamMessage(const void* const audioStreamMessage, int audioStreamMessageLength);

		/**
		 * Sets the URL of an audio stream.
		 */
		static void setAudioStreamURL(const void* const audioStreamURL, int audioStreamURLLength);
        
        /**
		 * Sets the name of an audio stream.
		 */
		static void setAudioStreamName(const void* const audioStreamName, int audioStreamNameLength);

		/**
		 * Sets the size in bytes of the audio track.
		 */
		static void setAudioTrackSizeInBytes(int audioTrackSizeInBytes);

		/**
		 * Sets the year of the audio track.
		 */
		static void setAudioTrackYear(int aYear);

		/**
		 * Sets whether the current audio data is a stream.
		 */
		static void setAudioDataIsStream(bool isStream);

		/**
		 * Analyzes the currently set audio track meta data.
		 * @return True if the track information changed (meaning: a new audio track started).
		 */
		static bool analyzeCurrentlySetMetadata(void);

		/**
		 * Sets the total time of the current audio track.
		 */
		static void setTotalTimeOfCurrentTrack(int totalTimeInMS);

		/**
		 * Sets the start time of the current audio track.
		 */
		static void setStartTimeOfCurrentTrack(int startTimeInMS);

		/**
		 * Sets the stop time of the current audio track.
		 */
		static void setStopTimeOfCurrentTrack(int stopTimeInMS);

		/**
		 * Posts the track info string of the current audio data.
		 */
		static void postTrackInfoOfCurrentAudioData(void);

		/**
		 * Posts the lyrics of the current audio data.
		 * @return True on success, false on failure.
		 */
		static bool postLyricsOfCurrentAudioData(void);

        /**
		 * Requests the album cover artwork of the currently playing audio.
		 */
        static void requestAlbumCoverArtworkOfCurrentAudioData(void);
        
		/**
		 * Posts the album cover artwork of the current audio data.
         * @param coverArtImageData A platform-specific pointer to the cover art data. On the Mac, this is a CFDataRef. On Windows, it is a LPCVOID pointer. The data is disposed after this call returns so the plugin must not save off the pointer and expect it to be valid later. If the plugin wants to keep the data around for later, it needs to retain it (Mac) or copy it (Windows). If the currently playing track has no artwork, this pointer will be nil.
         * @param coverArtDataSizeInBytes The size of the image data in bytes.
         * @param coverArtImageFormat The format of the image data (kVisualCoverArtFormatJPEG = 13, kVisualCoverArtFormatPNG = 14, kVisualCoverArtFormatBMP = 27).
		 */
		static void postAlbumCoverArtworkOfCurrentAudioData(VISUAL_PLATFORM_DATA coverArtImageData, uint32 coverArtDataSizeInBytes, uint32 coverArtImageFormat);

		/**
		 * Writes a 8-bit string to the system log file.
		 * @param infoStr The 8-bit string to write to log file.
		 */
		static void writeLogEntry(const char* const infoStr);

	private:

		/** The constructor. VisualMainAction is a collection of static methods. Class does not need any instances. Constructor is private and not implemented. */
		VisualMainAction();
		
		/** The destructor. VisualMainAction is a collection of static methods. Class does not need any instances. Destructor is private and not implemented. */
		~VisualMainAction();
        
        /* Collected setProcessInfo calls for monitoring the rendering. */
        static void monitorRenderMessageProcess();

	};

}

#endif /* VisualMainAction_h */
