/*
 * Project: VizKit
 * Version: 3.3.8
 
 * Date: 20150503
 * File: VisualNotificationQueue.cpp
 *
 */

/***************************************************************************

Copyright (c) 2004-2015 Heiko Wichmann (http://www.imagomat.de/vizkit)


This software is provided 'as-is', without any expressed or implied warranty. 
In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; 
   you must not claim that you wrote the original software. 
   If you use this software in a product, an acknowledgment 
   in the product documentation would be appreciated 
   but is not required.

2. Altered source versions must be plainly marked as such, 
   and must not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source distribution.

 ***************************************************************************/

#include "VisualNotificationQueue.h"
#include "VisualThreading.h"
#include "VisualThreadingManager.h"
#include "VisualErrorHandling.h"


#if TARGET_OS_MAC
#include <CoreServices/../Frameworks/CarbonCore.framework/Headers/Debugging.h> // GetMacOSStatusErrorString
#endif


using namespace VizKit;


VisualNotificationQueue* VisualNotificationQueue::visualNotificationQueueInstance = NULL;


VisualNotificationQueue::VisualNotificationQueue() {
	// null
}


VisualNotificationQueue::~VisualNotificationQueue() {
	notificationQueue.clear();
	inputNotificationQueue.clear();
}


VisualNotificationQueue* VisualNotificationQueue::getInstance() {
	if (VisualNotificationQueue::visualNotificationQueueInstance == NULL) {
        VisualThreadingManager::enterCriticalSection(VisualThreadingManager::notificationQueueSingletonMutex);
		if (VisualNotificationQueue::visualNotificationQueueInstance == NULL) {
			VisualNotificationQueue::visualNotificationQueueInstance = new VisualNotificationQueue;
			if (VisualNotificationQueue::visualNotificationQueueInstance == NULL) {
				char errLog[512];
				sprintf(errLog, "ERR: init VisualNotificationQueue failed in file: %s (line: %d) [%s])", __FILE__, __LINE__, __FUNCTION__);
				writeLog(errLog);
			}
		}
        VisualThreadingManager::exitCriticalSection(VisualThreadingManager::notificationQueueSingletonMutex);
	}
	return VisualNotificationQueue::visualNotificationQueueInstance;
}


void VisualNotificationQueue::dispose() {
	if (VisualNotificationQueue::visualNotificationQueueInstance != NULL) {
		delete VisualNotificationQueue::visualNotificationQueueInstance;
		VisualNotificationQueue::visualNotificationQueueInstance = NULL;
	}
}


void VisualNotificationQueue::push(VisualNotification& aNotification) {
	
	VisualNotificationQueue* theVisualNotificationQueue = VisualNotificationQueue::getInstance();
	
    VisualThreadingManager::enterCriticalSection(VisualThreadingManager::inputNotificationQueueAccessMutex);
	theVisualNotificationQueue->inputNotificationQueue.push_back(aNotification);
    VisualThreadingManager::exitCriticalSection(VisualThreadingManager::inputNotificationQueueAccessMutex);

}


VisualNotification VisualNotificationQueue::pop() {
	VisualNotification aNotification;
	VisualNotificationQueue* theVisualNotificationQueue = VisualNotificationQueue::getInstance();

    VisualThreadingManager::enterCriticalSection(VisualThreadingManager::notificationQueueAccessMutex);
	aNotification = theVisualNotificationQueue->notificationQueue.front();
	theVisualNotificationQueue->notificationQueue.pop_front();
    VisualThreadingManager::exitCriticalSection(VisualThreadingManager::notificationQueueAccessMutex);
	
	return aNotification;
}


size_t VisualNotificationQueue::size() {
	
	VisualNotificationQueue* theVisualNotificationQueue = VisualNotificationQueue::getInstance();

    VisualThreadingManager::enterCriticalSection(VisualThreadingManager::notificationQueueAccessMutex);
	size_t aSize = theVisualNotificationQueue->notificationQueue.size();
    VisualThreadingManager::exitCriticalSection(VisualThreadingManager::notificationQueueAccessMutex);

    return aSize;
}


void VisualNotificationQueue::update() {

	VisualNotificationQueue* theVisualNotificationQueue = VisualNotificationQueue::getInstance();

	// inputNotificationQueue
    VisualThreadingManager::enterCriticalSection(VisualThreadingManager::inputNotificationQueueAccessMutex);

	// notificationQueue
    VisualThreadingManager::enterCriticalSection(VisualThreadingManager::notificationQueueAccessMutex);

	for (NotificationQueueIterator it = theVisualNotificationQueue->inputNotificationQueue.begin(); it != theVisualNotificationQueue->inputNotificationQueue.end(); it++) {
		theVisualNotificationQueue->notificationQueue.push_back(*it);
	}
	theVisualNotificationQueue->inputNotificationQueue.clear();

	// notificationQueue
    VisualThreadingManager::exitCriticalSection(VisualThreadingManager::notificationQueueAccessMutex);

	// inputNotificationQueue
    VisualThreadingManager::exitCriticalSection(VisualThreadingManager::inputNotificationQueueAccessMutex);
}
